import React, { useState, useEffect } from 'react';
import ReactDOM from 'react-dom';
import { animate, json, scaleLinear, select, scaleBand, max } from 'd3';

const jsonUrl = 'https://gist.githubusercontent.com/SiddharthMittal07/3c7afd2a88faa690291f9474b60483b6/raw/c52aa3e8f2b870591d5fccd0957f236ee6141452/major_project_algorithms.json';

const width = 820;
const height = 450;
const margin = { top: 40, right: 20, left: 80, bottom: 65 };

const keyValue = [
  { value: 'testing_accuracy', label: 'Accuracy' },
  { value: 'time', label: 'Time' },
  { value: 'precision_score', label: 'Precision' },
  { value: 'recall_score', label: 'Recall' }
];

const trainTest = { 'testing_accuracy': 2, 'time': 1, 'precision_score': 1, 'recall_score': 1 };

const colorArray = ['#FF6633', '#FFB399', '#FF33FF', '#FFFF99', '#00B3E6', 
		  '#E6B333', '#3366E6', '#999966', '#99FF99', '#B34D4D',
		  '#80B300', '#809900', '#E6B3B3', '#6680B3', '#66991A', 
		  '#FF99E6', '#CCFF1A', '#FF1A66', '#E6331A', '#33FFCC',
		  '#66994D', '#B366CC', '#4D8000', '#B33300', '#CC80CC', 
		  '#66664D', '#991AFF', '#E666FF', '#4DB3FF', '#1AB399',
		  '#E666B3', '#33991A', '#CC9999', '#B3B31A', '#00E680', 
		  '#4D8066', '#809980', '#E6FF80', '#1AFF33', '#999933',
		  '#FF3380', '#CCCC00', '#66E64D', '#4D80CC', '#9900B3', 
		  '#E64D66', '#4DB380', '#FF4D4D', '#99E6E6', '#6666FF'];

const App = () => {
	const [data, setData] = useState(null);
  const initialYAttribute = 'precision_score';
  const [yAttribute, setYAttribute] = useState(initialYAttribute);
  
  const getLabel = (value) => {
   	for(let i = 0; i < keyValue.length; i++) {
     	if (keyValue[i].value === value) {
        return keyValue[i].label;
      }
    }
  }
  
  useEffect(() => {
  	json(jsonUrl).then(setData);
  }, []);

  if (!data) {
    return <div>Loading...</div>;
  }
  
  const innerHeight = height - margin.top - margin.bottom;
  const innerWidth = width - margin.left - margin.right;
  
  const yValue = d => d[yAttribute]
  
  const xScale = scaleBand().domain(data.map(d => d.algorithm)).range([0, innerWidth]).padding(0.5);
  const yScale = scaleLinear().domain([0, max(data, yValue)]).range([innerHeight, 0]);
  
  return (
  	<div className='panel'>
      <svg width={width} height={height}>
      	<g transform={`translate(${margin.left},${margin.top})`}>
          {
          	xScale.domain().map(tickValue => <g className='tick' transform={`translate(${xScale(tickValue)},${innerHeight + 3})`}>
                                  <text
                                    style={{textAnchor: 'middle'}}
                                    dy="0.71em"
                                    >{tickValue}</text>
                                </g>)
          }
          <g transform={`translate(0,${innerHeight})`}>
          	<line 
              x2={innerWidth}
              stroke='black'
              />
          </g>
          {
          	yScale.ticks().map(tickValue => <g className='tick' transform={`translate(-5,${yScale(tickValue)})`}>
                                 <text
                                   style={{textAnchor: 'end'}}
                                   dy="0.32em"
                                   >{tickValue}</text>  
                                 <line 
                                   x2={10}
                                   stroke='black'
                                   />
                               </g>)
          }
          <g>
            <line 
              y2={innerHeight}
              stroke='black'
              />
          </g>
          {
          	data.map(d => <g 
                            className='bar'
                            transform={`translate(${xScale(d.algorithm)}, ${yScale(yValue(d))})`}
                            onMouseEnter={() => {
                         Object.keys(trainTest).map((key) => 
                                                    select('#tooltip').append('span').text(getLabel(key) + ' : ' + d[key]) 
                                                    );
                         select('#tooltip').style('top', innerHeight - 100 + 'px');
                         select('#tooltip').style('left', xScale(d.algorithm) + margin.left + 'px');
                       	select('#tooltip').style('visibility','visible');
                       }}
                            onMouseLeave={() => {
                       		select('#tooltip').selectAll('span').remove();
                         select('#tooltip').style('visibility','hidden');
                       }}
                            >
                    	<rect
                        x={-xScale.bandwidth()/2}
                        width={xScale.bandwidth()}
                        height={innerHeight - yScale(yValue(d))}
                        fill={colorArray[Math.floor(Math.random() * colorArray.length)]}
                        >
                        <animate attributeName="height" dur="3s" from={yScale(yValue(d))} to={innerHeight - yScale(yValue(d))}/>
                       </rect>  
                     </g>)
          }
          <text
            id='axis-label'
            transform={`translate(-${margin.left / 2},${innerHeight/2}) rotate(-90)`}
            style={{textAnchor: 'middle'}}
            >{getLabel(yAttribute)}</text>
        </g>
      </svg>
      <div id='tooltip'>
      </div>
      <div id='options'>
        <label for='choose-factor'>Factor:</label>
        <select id='choose-factor' onChange={(e) => setYAttribute(e.target.value)}>
          {
          	keyValue.map(({value, label}) => <option
                                               value={value}
                                               selected={value === yAttribute}
                                               >{label}</option>)
          }
        </select>
      </div>
    </div>
  );
}

ReactDOM.render(<App />, document.getElementById('root'));